#!/usr/bin/env python3


def decorate_actual_iv( iv, signal=None ):
    orig_iv_name = iv["class"]
    iv_name = orig_iv_name
    
    if signal:
        iv_name = f"{signal}->{iv_name}"
    if orig_iv_name == "PropertyValueChanger":
        key = iv["Target_Property_Key"]
        value = iv["Target_Property_Value"]
        iv_name += f"({key}:{value})"
    elif orig_iv_name == "MigrateIndividuals":
        dest = iv["NodeID_To_Migrate_To"]
        iv_name += f"({dest})"
    elif orig_iv_name == "DelayedIntervention":
        key = "Delay_Period_Distribution" if "Delay_Period_Distribution" in iv else "Delay_Distribution"
        dist = iv[key].replace("_DURATION","")
        iv_name += f"({dist})"
    elif orig_iv_name == "AntimalarialDrug":
        drug = iv["Drug_Type"]
        iv_name += f"({drug})"
    elif orig_iv_name == "BroadcastEvent":
        signal = iv["Broadcast_Event"]
        iv_name += f"({signal})"
    elif orig_iv_name == "HIVRandomChoice":
        choices = iv["Choices"]
        iv_name += f"({choices})"
    elif orig_iv_name == "HIVRapidHIVDiagnostic":
        pos = iv["Positive_Diagnosis_Event"]
        neg = iv["Negative_Diagnosis_Event"]
        iv_name += f"({pos}/{neg})"
    elif orig_iv_name == "SimpleHealthSeekingBehavior":
        if "Intervention_Name" in iv:
            iv_name = iv["Intervention_Name"]
        goal = iv["New_Property_Value"]
        iv_name += f"({goal})"
    return iv_name

def handle_di( iv ):
    new_iv_name = "."
    for act_iv in iv["Actual_IndividualIntervention_Configs"]:
        new_iv_name += decorate_actual_iv( act_iv )
        new_iv_name += "+"
    return new_iv_name.strip( "+" )

def decode( camp_path ):
    import json
    cj = json.load( open( camp_path ) )
    print( f"{ len( cj['Events'] ) }" )
    for event in cj['Events']:
        frac = 1
        sex = "Both"
        ip = ""
        try:
            # when
            if 'Start_Day' in event:
                day = int(event['Start_Day'])
            else:
                day = 1
            coord = event['Event_Coordinator_Config']
            if coord["class"] != "StandardInterventionDistributionEventCoordinator":
                #print( f"DEBUG: {coord['class']} not fully supported yet." )
                frac = "???"
                #continue
            if 'Number_Repetitions' in coord and coord['Number_Repetitions'] != 1:
                reps = coord['Number_Repetitions']
                gap = None
                if coord['Timesteps_Between_Repetitions'] != -1:
                    gap = coord['Timesteps_Between_Repetitions']
                day = f"{day}(x{reps}/_{gap})"

            # where
            nc = event['Nodeset_Config']
            if nc["class"] == "NodeSetAll":
                nodes = "AllPlaces"
            else:
                nodes = nc["Node_List"]

        # who
            if "Demographic_Coverage" in coord:
                frac = coord["Demographic_Coverage"]
            if "Target_Gender" in coord:
                sex = coord["Target_Gender"]
            if "Property_Restrictions" in coord:
                ip = coord["Property_Restrictions"]
                if len(ip)==0:
                    ip = ""
                else:
                    ip = str(ip).strip("[").strip("]").strip("'").replace(":","=")

            # what
            iv = event['Event_Coordinator_Config']['Intervention_Config']
            iv_name = decorate_actual_iv( iv )
            if iv_name == "NodeLevelHealthTriggeredIV":
                signal = iv["Trigger_Condition_List"][0]
                if "Demographic_Coverage" in iv:
                    frac = iv["Demographic_Coverage"]
                iv = iv["Actual_IndividualIntervention_Config"]
                iv_name = decorate_actual_iv( iv, signal )
            if iv_name.split("->")[-1].startswith( "MultiInterventionDistributor" ):
                new_iv_name = "."
                for act_iv in iv["Intervention_List"]:
                    new_iv_name += decorate_actual_iv( act_iv )
                    new_iv_name += "|"
                iv_name += new_iv_name.strip( "|" )
                if iv_name.split("|")[-1].startswith( "DelayedIntervention" ): # ugly copy-paste
                    iv_name += handle_di( act_iv )
            if iv_name.split("->")[-1].startswith( "DelayedIntervention" ): # we should have a heuristic, or a list
                iv_name += handle_di( iv )

            who = "STEERED" if type(frac) is str else f"{float(frac)*100}%"
            if "ale" in sex:
                who += f"/{sex}"
            if ip != "":
                who += f"/{ip}"

            print( f"{day} :: {nodes} :: {who} :: {iv_name}" )

        except Exception as ex:
            print( "Exception..." )
            print( str( ex ) )
            print( str( event ) )

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--campaign', 
                help='Existing campaign.json file path.'
            )
    args = parser.parse_args()
    decode( args.campaign )
